import React, { useContext, useState, useEffect, useRef } from "react"
import { isAdministrator } from "../../../services/auth"
import { navigate } from "gatsby"
import { Table, Button, Form, Switch, notification } from "antd"
import { decodeUnicode } from "../../../utils/helper"

const EditableContext = React.createContext(null)

const EditableRow = ({ index, ...props }) => {
	const [form] = Form.useForm()
	return (
		<Form form={form} component={false}>
			<EditableContext.Provider value={form}>
				<tr {...props} />
			</EditableContext.Provider>
		</Form>
	)
}

const EditableCell = ({
	title,
	editable,
	children,
	inputType,
	dataIndex,
	record,
	handleEdit,
	...restProps
}) => {
	const [editing, setEditing] = useState(false)
	const inputRef = useRef(null)
	const form = useContext(EditableContext)
	useEffect(() => {
		if (editing) {
			inputRef.current.focus()
		}
	}, [editing])

	const toggleEdit = () => {
		setEditing(!editing)
		form.setFieldsValue({
			[dataIndex]: record[dataIndex],
		})
	}

	const saveEdit = async () => {
		try {
			const values = await form.validateFields()
			console.log("values ", values)
			toggleEdit()
			handleEdit({ ...record, ...values })
		} catch (errInfo) {
			console.log("Save edit failed:", errInfo)
		}
	}

	let childNode = children

	if (editable) {
		childNode = editing ? (
			<Form.Item
				style={{
					margin: 0,
				}}
				name={dataIndex}
				valuePropName="checked"
			>
				<Switch
					size="small"
					ref={inputRef}
					onChange={saveEdit}
					onBlur={saveEdit}
				/>
			</Form.Item>
		) : (
			<div
				className="editable-cell-value-wrap"
				style={{
					paddingRight: 24,
				}}
				role="button"
				tabIndex="0"
				onClick={toggleEdit}
				onKeyDown={toggleEdit}
			>
				{children}
			</div>
		)
	}

	return <td {...restProps}>{childNode}</td>
}

class EditableLanguages extends React.Component {
	constructor(props) {
		super(props)
		this.columns = [
			{
				title: "Title",
				dataIndex: "title",
				editable: false,
			},
			{
				title: "Path",
				dataIndex: "path",
				editable: false,
			},
			{
				title: "Enabled",
				dataIndex: "enabled",
				editable: true,
				render: (_, record) => {
					return <Switch defaultChecked={record.enabled} size="small" />
				},
			},
			{
				title: "Default",
				dataIndex: "default",
				editable: true,
				render: (_, record) => {
					return <Switch defaultChecked={record.default} size="small" />
				},
			},
		]
		this.state = {
			dataSource: null,
			sha: "",
		}
	}

	handleEdit = (row) => {
		const newData = [...this.state.dataSource]
		this.state.dataSource.map((root, i) => {
			if (root.key === row.key) {
				newData.splice(i, 1, row)
			}
			return root
		})
		this.setState({
			dataSource: newData,
		})
	}
	async handleGitGet() {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
					duration: 0,
				})
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	async handleGitUpdate(contentStr) {
		try {
			const jsonStr = JSON.stringify(contentStr)
			const data = {
				message:
					"[skip ci] Update @maad-dev/gatsby-theme-backend/settings/languages.json from Modern backend.",
				content: btoa(jsonStr),
				sha: this.state.sha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error fetchGitApi PUT ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
					duration: 0,
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	handleSave = () => {
		const { dataSource } = this.state
		console.log("data ", dataSource)
		/*this.handleGitUpdate(dataSource).then((updateFile) => {
			if (updateFile) {
				notification["success"]({
					message: "Success!",
					description: "Updated languages successfully.",
					duration: 0,
				})
			} else {
				let errMsg = "No status"
				if (updateFile === 404) {
					errMsg = "Status: 404 Not Found"
				} else if (updateFile === 409) {
					errMsg = "Status: 409 Conflict"
				} else if (updateFile === 422) {
					errMsg = "Status: 422 Unprocessable Entity"
				}
				notification["error"]({
					message: "Error!",
					description: errMsg,
					duration: 0,
				})
			}
		})*/
	}
	async componentDidMount() {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}

		await this.handleGitGet().then((getFile) => {
			if (getFile !== false) {
				const decodeContent = decodeUnicode(getFile.content)
				this.setState({
					dataSource: JSON.parse(decodeContent),
					sha: getFile.sha,
				})
			}
		})
	}

	render() {
		const { dataSource } = this.state
		const components = {
			body: {
				row: EditableRow,
				cell: EditableCell,
			},
		}
		const columns = this.columns.map((col) => {
			if (!col.editable) {
				return col
			}

			return {
				...col,
				onCell: (record) => ({
					record,
					editable: col.editable,
					inputType: "button",
					dataIndex: col.dataIndex,
					title: col.title,
					handleEdit: this.handleEdit,
				}),
			}
		})

		return (
			<div className="editableMenusContainer">
				<div className="topWrapper">
					<p>&nbsp;</p>
					<Button onClick={this.handleSave} type="primary">
						Save
					</Button>
				</div>
				<Table
					rowKey="key"
					components={components}
					rowClassName={() => "editable-row"}
					dataSource={dataSource}
					columns={columns}
					pagination={false}
				/>
			</div>
		)
	}
}

export default EditableLanguages
